@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004-2017 Jarosław Staniek <staniek@kde.org>
+   Copyright (C) 2004-2018 Jarosław Staniek <staniek@kde.org>
    Copyright (c) 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
    Copyright (c) 1999 Preston Brown <pbrown@kde.org>
    Copyright (c) 1997 Matthias Kalle Dalheimer <kalle@kde.org>
@@ -21,22 +21,23 @@
 */
 
 #include "KDb.h"
-#include "KDbConnectionData.h"
 #include "KDbConnection.h"
+#include "KDbConnectionData.h"
 #include "KDbCursor.h"
-#include "kdb_debug.h"
+#include "KDbDateTime.h"
 #include "KDbDriverBehavior.h"
 #include "KDbDriverManager.h"
 #include "KDbDriver_p.h"
 #include "KDbLookupFieldSchema.h"
 #include "KDbMessageHandler.h"
 #include "KDbNativeStatementBuilder.h"
-#include "KDb_p.h"
 #include "KDbQuerySchema.h"
 #include "KDbRecordData.h"
 #include "KDbSqlResult.h"
 #include "KDbTableOrQuerySchema.h"
 #include "KDbVersionInfo.h"
+#include "KDb_p.h"
+#include "kdb_debug.h"
 #include "transliteration/transliteration_table.h"
 
 #include <QMap>
@@ -1325,14 +1326,21 @@ inline static T escapeIdentifier(const T& string, bool addQuotes)
     return newString;
 }
 
+static bool shouldAddQuotesToIdentifier(const QByteArray& string)
+{
+    return !string.isEmpty() && (!KDb::isIdentifier(string) || KDb::isKDbSqlKeyword(string));
+}
+
 QString KDb::escapeIdentifier(const QString& string)
 {
-    return ::escapeIdentifier<QString, QLatin1String, QLatin1Char, QChar>(string, false);
+    return ::escapeIdentifier<QString, QLatin1String, QLatin1Char, QChar>(
+        string, shouldAddQuotesToIdentifier(string.toLatin1()));
 }
 
 QByteArray KDb::escapeIdentifier(const QByteArray& string)
 {
-    return ::escapeIdentifier<QByteArray, QByteArray, char, char>(string, false);
+    return ::escapeIdentifier<QByteArray, QByteArray, char, char>(
+        string, shouldAddQuotesToIdentifier(string));
 }
 
 QString KDb::escapeIdentifierAndAddQuotes(const QString& string)
@@ -1622,7 +1630,7 @@ QString KDb::escapeBLOB(const QByteArray& array, BLOBEscapingType type)
     int new_length = str.length(); //after X' or 0x, etc.
     if (type == BLOBEscapingType::Octal) {
         // only escape nonprintable characters as in Table 8-7:
-        // http://www.postgresql.org/docs/8.1/interactive/datatype-binary.html
+        // https://www.postgresql.org/docs/8.1/interactive/datatype-binary.html
         // i.e. escape for bytes: < 32, >= 127, 39 ('), 92(\).
         for (int i = 0; i < size; i++) {
             const unsigned char val = array[i];
@@ -1680,7 +1688,7 @@ QByteArray KDb::pgsqlByteaToByteArray(const char* data, int length)
         for (int input = 0; s < end; output++) {
             //  kdbDebug()<<(int)s[0]<<" "<<(int)s[1]<<" "<<(int)s[2]<<" "<<(int)s[3]<<" "<<(int)s[4];
             if (s[0] == '\\' && (s + 1) < end) {
-                //special cases as in http://www.postgresql.org/docs/8.1/interactive/datatype-binary.html
+                //special cases as in https://www.postgresql.org/docs/8.1/interactive/datatype-binary.html
                 if (s[1] == '\'') {// \'
                     if (pass == 1)
                         array[output] = '\'';
@@ -2196,11 +2204,114 @@ QString KDb::identifierExpectedMessage(const QString &valueName, const QVariant&
            + kdb::tr("\"%1\" is not a valid identifier.").arg(v.toString()) + QLatin1String("</p>");
 }
 
+//---------
+
+KDbEscapedString KDb::valueToSql(KDbField::Type ftype, const QVariant& v)
+{
+    return valueToSqlInternal(nullptr, ftype, v);
+}
+
+static QByteArray dateToSqlInternal(const QVariant& v, bool allowInvalidKDbDate)
+{
+    QByteArray result(QByteArrayLiteral("<INVALID_DATE>"));
+    if (v.canConvert<KDbDate>()) {
+        const KDbDate date(v.value<KDbDate>());
+        if (date.isValid() || allowInvalidKDbDate) {
+            result = date.toString(); // OK even if invalid or null
+        }
+    } else if (v.canConvert<QDate>()) {
+        const QDate date(v.toDate());
+        if (date.isValid()) {
+            result = date.toString(Qt::ISODate).toLatin1();
+        }
+    }
+    return result;
+}
+
+KDbEscapedString KDb::dateToSql(const QVariant& v)
+{
+    return KDbEscapedString('#') + dateToSqlInternal(v, true) + '#';
+}
+
+static QByteArray timeToSqlInternal(const QVariant& v, bool allowInvalidKDbTime)
+{
+    QByteArray result(QByteArrayLiteral("<INVALID_TIME>"));
+    if (v.canConvert<KDbTime>()) {
+        const KDbTime time(v.value<KDbTime>());
+        if (time.isValid() || allowInvalidKDbTime) {
+            result = time.toString(); // OK even if invalid or null
+        }
+    } else if (v.canConvert<QTime>()) {
+        const QTime time(v.toTime());
+        if (time.isValid()) {
+            if (time.msec() == 0) {
+                result = time.toString(Qt::ISODate).toLatin1();
+            } else {
+                result = time.toString(Qt::ISODateWithMs).toLatin1();
+            }
+        }
+    }
+    return result;
+}
+
+KDbEscapedString KDb::timeToSql(const QVariant& v)
+{
+    return KDbEscapedString('#') + timeToSqlInternal(v, true) + '#';
+}
+
+static QByteArray dateTimeToSqlInternal(const QVariant& v, char separator, bool allowInvalidKDbDateTime)
+{
+    QByteArray result(QByteArrayLiteral("<INVALID_DATETIME>"));
+    if (v.canConvert<KDbDateTime>()) {
+        const KDbDateTime dateTime(v.value<KDbDateTime>());
+        if (dateTime.isValid() || allowInvalidKDbDateTime) {
+            result = dateTime.toString(); // OK even if invalid or null
+        }
+    } else if (v.canConvert<QDateTime>()) {
+        const QDateTime dateTime(v.toDateTime());
+        if (dateTime.isValid()) {
+            result = dateTime.date().toString(Qt::ISODate).toLatin1() + separator;
+            const QTime time(dateTime.time());
+            if (time.msec() == 0) {
+                result += time.toString(Qt::ISODate).toLatin1();
+            } else {
+                result += time.toString(Qt::ISODateWithMs).toLatin1();
+            }
+        }
+    }
+    return result;
+}
+
+KDbEscapedString KDb::dateTimeToSql(const QVariant& v)
+{
+    return KDbEscapedString('#') + dateTimeToSqlInternal(v, ' ', true) + '#';
+}
+
+KDbEscapedString KDb::dateTimeToSql(const QDateTime& v)
+{
+    return KDb::dateTimeToIsoString(v);
+}
+
+KDbEscapedString KDb::dateToIsoString(const QVariant& v)
+{
+    return KDbEscapedString('\'') + dateToSqlInternal(v, false) + KDbEscapedString('\'');
+}
+
+KDbEscapedString KDb::timeToIsoString(const QVariant& v)
+{
+    return KDbEscapedString('\'') + timeToSqlInternal(v, false) + KDbEscapedString('\'');
+}
+
+KDbEscapedString KDb::dateTimeToIsoString(const QVariant& v)
+{
+    return KDbEscapedString('\'') + dateTimeToSqlInternal(v, 'T', false) + KDbEscapedString('\'');
+}
+
 //--------------------------------------------------------------------------------
 
 #ifdef KDB_DEBUG_GUI
 
-static KDb::DebugGUIHandler s_debugGUIHandler = 0;
+static KDb::DebugGUIHandler s_debugGUIHandler = nullptr;
 
 void KDb::setDebugGUIHandler(KDb::DebugGUIHandler handler)
 {
@@ -2213,7 +2324,7 @@ void KDb::debugGUI(const QString& text)
         s_debugGUIHandler(text);
 }
 
-static KDb::AlterTableActionDebugGUIHandler s_alterTableActionDebugHandler = 0;
+static KDb::AlterTableActionDebugGUIHandler s_alterTableActionDebugHandler = nullptr;
 
 void KDb::setAlterTableActionDebugHandler(KDb::AlterTableActionDebugGUIHandler handler)
 {
